@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2006 Sun Microsystems Inc.
+# Copyright 2005-2007 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -19,9 +19,9 @@
 
 """Provides the interface to the AT-SPI Registry."""
 
-__id__        = "$Id: atspi.py 1379 2006-08-28 13:52:17Z oserb $"
-__version__   = "$Revision: 1379 $"
-__date__      = "$Date: 2006-08-28 09:52:17 -0400 (Mon, 28 Aug 2006) $"
+__id__        = "$Id: atspi.py 1925 2007-01-15 02:32:55Z wwalker $"
+__version__   = "$Revision: 1925 $"
+__date__      = "$Date: 2007-01-14 21:32:55 -0500 (Sun, 14 Jan 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -64,31 +64,6 @@ class Event:
     object with their own attributes.
     """
 
-    def getAnyData(e):
-        """Returns the any_data field, compensating for any
-        differences between pre-1.7.0 and post-1.7.0 implementations
-        of the AT-SPI."""
-
-        # [[[TODO: WDW - HACK because AT-SPI 1.7.0 has
-        # introduced a binary incompatibility where the
-        # "any_data" of the event has changed from being a
-        # real "any_data" to an EventDetails structure that
-        # holds "any_data."  This check here helps us deal
-        # with the case where we might be talking to a
-        # pre-1.7.0 implementation or a 1.7.0+ implementation.
-        # What we're doing is just checking the CORBA typecode
-        # (as a string) to see if the type of the "any_data"
-        # field is this new structure.  If it is, we pull the
-        # "any_data" field from it.  If it isn't, we get the
-        # "any_data" field as we normally would.]]]
-        #
-        if e.any_data and (e.any_data.typecode().name) == "EventDetails":
-            return e.any_data.value().any_data
-        else:
-            return e.any_data
-
-    getAnyData = staticmethod(getAnyData)
-
     def __init__(self, e=None):
         if e:
             self.source   = Accessible.makeAccessible(e.source)
@@ -108,6 +83,21 @@ class Event:
                         details.host_application)
             else:
                 self.any_data = e.any_data
+
+            # We need to make sure we reference any object that comes
+            # to us via an any_data because we process events
+            # asynchronously.  If we don't reference them, we may
+            # end up with OBJECT_NOT_EXIST errors.  Please see
+            # http://bugzilla.gnome.org/show_bug.cgi?id=395749 for
+            # more information.
+            #
+            if self.type == "object:active-descendant-changed":
+                self.any_data = Accessible.makeAccessible(
+                    self.any_data.value())
+            elif self.type == "object:text-changed:insert":
+                self.any_data = self.any_data.value()
+            elif self.type == "object:text-changed:delete":
+                self.any_data = self.any_data.value()
         else:
             self.source   = None
             self.type     = None
@@ -163,7 +153,7 @@ class Registry:
             debug.println(debug.LEVEL_CONFIGURATION,
                           "atspi.start: using bonobo.main; "
                           + "gilSleepTime=%f" % settings.gilSleepTime)
-            if settings.gilSleepTime:
+            if settings.useBlockPreventor and settings.gilSleepTime:
                 gobject.idle_add(self.__blockPreventor)
             bonobo.main()
         else:
@@ -683,6 +673,52 @@ class Accessible:
             except:
                 debug.printException(debug.LEVEL_SEVERE)
 
+    def getRelationString(self):
+        """Returns a space-delimited string composed of the given object's
+        Accessible relations attribute.  This is for debug purposes.
+        """
+
+        relations = self.relations
+        relString = " "
+        for relation in relations:
+            if relation.getRelationType() == Accessibility.RELATION_LABEL_FOR:
+                relString += "LABEL_FOR "
+            if relation.getRelationType() == Accessibility.RELATION_LABELLED_BY:
+                relString += "LABELLED_BY "
+            if relation.getRelationType() == \
+                                       Accessibility.RELATION_CONTROLLER_FOR:
+                relString += "CONTROLLER_FOR "
+            if relation.getRelationType() == \
+                                       Accessibility.RELATION_CONTROLLED_BY:
+                relString += "CONTROLLED_BY "
+            if relation.getRelationType() == Accessibility.RELATION_MEMBER_OF:
+                relString += "MEMBER_OF "
+            if relation.getRelationType() == Accessibility.RELATION_TOOLTIP_FOR:
+                relString += "TOOLTIP_FOR "
+            if relation.getRelationType() == \
+                                       Accessibility.RELATION_NODE_CHILD_OF:
+                relString += "NODE_CHILD_OF "
+            if relation.getRelationType() == Accessibility.RELATION_EXTENDED:
+                relString += "RELATION_EXTENDED "
+            if relation.getRelationType() == Accessibility.RELATION_FLOWS_TO:
+                relString += "FLOWS_TO "
+            if relation.getRelationType() == Accessibility.RELATION_FLOWS_FROM:
+                relString += "FLOWS_FROM "
+            if relation.getRelationType() == \
+                                       Accessibility.RELATION_SUBWINDOW_OF:
+                relString += "SUBWINDOW_OF "
+            if relation.getRelationType() == Accessibility.RELATION_EMBEDS:
+                relString += "EMBEDS "
+            if relation.getRelationType() == Accessibility.RELATION_EMBEDDED_BY:
+                relString += "EMBEDDED_BY "
+            if relation.getRelationType() == Accessibility.RELATION_POPUP_FOR:
+                relString += "POPUP_FOR "
+            if relation.getRelationType() == \
+                                       Accessibility.RELATION_PARENT_WINDOW_OF:
+                relString += "WINDOW_OF "
+
+        return relString.strip()
+
     def getStateString(self):
         """Returns a space-delimited string composed of the given object's
         Accessible state attribute.  This is for debug purposes.
@@ -872,6 +908,21 @@ class Accessible:
 
         role = self.accessible.getRoleName()
 
+        # [[[TODO: WDW - HACK to handle the situation where some
+        # things might not be quite lined up with the ATK and AT-SPI.
+        # That is, some roles might have ids but no string yet.  See
+        # http://bugzilla.gnome.org/show_bug.cgi?id=361757.
+        #
+        if not len(role):
+            try:
+                roleId = self.accessible.getRole()
+                if roleId == Accessibility.ROLE_LINK:
+                    role = rolenames.ROLE_LINK
+                elif roleId == Accessibility.ROLE_INPUT_METHOD_WINDOW:
+                    role = rolenames.ROLE_INPUT_METHOD_WINDOW
+            except:
+                pass
+
         # [[[TODO: HACK to coalesce menu items with children into
         # menus.  The menu demo in gtk-demo does this, and one
         # might view that as an edge case.  But, in
@@ -966,9 +1017,6 @@ class Accessible:
             except:
                 pass
 
-        if settings.cacheValues:
-            self.relations = relations
-
         return relations
 
     def __get_app(self):
@@ -1091,6 +1139,26 @@ class Accessible:
 
         return component
 
+    def __get_hyperlink(self):
+        """Returns an object that implements the Accessibility_Hyperlink
+        interface for this object, or None if this object doesn't implement
+        the Accessibility_Hyperlink interface.
+        """
+
+        hyperlink = self.accessible.queryInterface(\
+            "IDL:Accessibility/Hyperlink:1.0")
+
+        if hyperlink:
+            try:
+                hyperlink = hyperlink._narrow(Accessibility.Hyperlink)
+            except:
+                hyperlink = None
+
+        if hyperlink and settings.cacheValues:
+            self.hyperlink = hyperlink
+
+        return hyperlink
+
     def __get_hypertext(self):
         """Returns an object that implements the Accessibility_Hypertext
         interface for this object, or None if this object doesn't implement
@@ -1208,6 +1276,17 @@ class Accessible:
 
         return value
 
+    def __get_attributes(self):
+        """Returns an Accessibility_AttributeSet of the object.
+        """
+
+        try:
+            attributes = self.accessible.getAttributes()
+        except:
+            attributes = None
+
+        return attributes
+
     def __getattr__(self, attr):
         """Created virtual attributes for the Accessible object to make
         the syntax a bit nicer (e.g., acc.name rather than acc.name()).
@@ -1247,6 +1326,8 @@ class Accessible:
             return self.__get_action()
         elif attr == "component":
             return self.__get_component()
+        elif attr == "hyperlink":
+            return self.__get_hyperlink()
         elif attr == "hypertext":
             return self.__get_hypertext()
         elif attr == "image":
@@ -1259,6 +1340,8 @@ class Accessible:
             return self.__get_text()
         elif attr == "value":
             return self.__get_value()
+        elif attr == "attributes":
+            return self.__get_attributes()
         elif attr.startswith('__') and attr.endswith('__'):
             raise AttributeError, attr
         else:
@@ -1291,7 +1374,13 @@ class Accessible:
                 newChild.app = self.app
 
         if not newChild:
-            raise Exception("Invalid child at index: %d" % index)
+            # The problem with a child not existing is a bad one.
+            # We want to issue a warning and we also want to know
+            # where it happens.
+            #
+            debug.printStack(debug.LEVEL_WARNING)
+            debug.println(debug.LEVEL_WARNING,
+                          "Child at index %d is not an Accessible" % index)
 
         return newChild
 
@@ -1315,10 +1404,11 @@ class Accessible:
         else:
             string = indent
 
-        string += "name=%s role='%s' state='%s'" \
+        string += "name=%s role='%s' state='%s' relations='%s'" \
                   % (self.accessibleNameToString(),
                      self.role,
-                     self.getStateString())
+                     self.getStateString(),
+                     self.getRelationString())
 
         return string
 
@@ -1349,7 +1439,7 @@ def __printDesktops():
         for j in range(0, desktop.childCount):
             app = desktop.getChildAtIndex(j)
             print "    App %d: name=%s role=%s" \
-                  % (j, app.name, app.role)
+                  % (j, app.name, app.getRoleName())
 
 def __notifyEvent(event):
         print event.type, event.source.name, \
