@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2006 Sun Microsystems Inc.
+# Copyright 2005-2007 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 1711 2006-11-16 19:11:58Z wwalker $"
-__version__   = "$Revision: 1711 $"
-__date__      = "$Date: 2006-11-16 14:11:58 -0500 (Thu, 16 Nov 2006) $"
+__id__        = "$Id: braillegenerator.py 1933 2007-01-17 19:32:35Z wwalker $"
+__version__   = "$Revision: 1933 $"
+__date__      = "$Date: 2007-01-17 14:32:35 -0500 (Wed, 17 Jan 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1086,13 +1086,16 @@ class BrailleGenerator:
         and the second element is the Region which should get focus.
         """
 
+        self._debugGenerator("_getBrailleRegionsForTableColumnHeader", obj)
+
         regions = []
 
         # Adding in a check here to make sure that the parent is a
         # valid table. It's possible that the parent could be a
         # table cell too (see bug #351501).
         #
-        if settings.readTableCellRow and obj.parent.table:
+        if settings.readTableCellRow and obj.parent.table \
+            and (not self._script.isLayoutOnly(obj.parent)):
             rowRegions = []
             savedBrailleVerbosityLevel = settings.brailleVerbosityLevel
             settings.brailleVerbosityLevel = \
@@ -1400,17 +1403,16 @@ class BrailleGenerator:
             #
             if (parent.role != rolenames.ROLE_FILLER) \
                 and (parent.role != rolenames.ROLE_SECTION) \
-                and (parent.role != rolenames.ROLE_SPLIT_PANE):
+                and (parent.role != rolenames.ROLE_SPLIT_PANE) \
+                and (not self._script.isLayoutOnly(parent)):
 
                 # Announce the label and text of the object in the hierarchy.
                 #
                 label = util.getDisplayedLabel(parent)
                 text = util.getDisplayedText(parent)
-                if ((label and len(label)) or (text and len(text))) \
-                    or (parent.role != rolenames.ROLE_PANEL):
-                    regions.append(braille.Region(" "))
-                    result = self.getBrailleRegions(parent, False)
-                    regions.extend(result[0])
+                regions.append(braille.Region(" "))
+                result = self.getBrailleRegions(parent, False)
+                regions.extend(result[0])
 
             # [[[TODO: HACK - we've discovered oddness in hierarchies
             # such as the gedit Edit->Preferences dialog.  In this
