@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Sun Microsystems Inc.
+# Copyright 2005-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -23,9 +23,9 @@ MouseButtonEvent, MouseMotionEvent, and SpeechEvent), and also provides a
 InputEventHandler class.  It is intended that instances of InputEventHandler
 will be used which should be used to handle all input events."""
 
-__id__        = "$Id: input_event.py,v 1.14 2006/09/12 13:45:16 wwalker Exp $"
-__version__   = "$Revision: 1.14 $"
-__date__      = "$Date: 2006/09/12 13:45:16 $"
+__id__        = "$Id: input_event.py 1529 2006-10-05 16:31:04Z wwalker $"
+__version__   = "$Revision: 1529 $"
+__date__      = "$Date: 2006-10-05 12:31:04 -0400 (Thu, 05 Oct 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -67,7 +67,7 @@ class KeyboardEvent(InputEvent):
         # be turned into the string "A").  All these checks here are
         # to just do some sanity checking before doing the
         # conversion. [[[WDW - this is making assumptions about
-        # mapping ASCII control characters to to UTF-8.]]]
+        # mapping ASCII control characters to UTF-8.]]]
         #
         event_string = event.event_string
         if (event.modifiers & (1 << atspi.Accessibility.MODIFIER_CONTROL)) \
