@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2006 Sun Microsystems Inc.
+# Copyright 2006-2007 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -24,9 +24,9 @@
 
 """Displays a GUI for the user to set Orca preferences."""
 
-__id__        = "$Id: orca_gui_prefs.py 1872 2007-01-03 17:33:35Z richb $"
-__version__   = "$Revision: 1872 $"
-__date__      = "$Date: 2007-01-03 12:33:35 -0500 (Wed, 03 Jan 2007) $"
+__id__        = "$Id: orca_gui_prefs.py 1952 2007-01-21 22:42:11Z wwalker $"
+__version__   = "$Revision: 1952 $"
+__date__      = "$Date: 2007-01-21 17:42:11 -0500 (Sun, 21 Jan 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -384,7 +384,7 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
         else:
             self.speechVerboseButton.set_active(True)
 
-        if prefs["readTableCellRow"] == True:
+        if prefs["readTableCellRow"]:
             self.rowSpeechButton.set_active(True)
         else:
             self.cellSpeechButton.set_active(True)
@@ -655,14 +655,16 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
         if len(self.families) > 1:
             i = 1
             for family in self.families:
-                name = family[speechserver.VoiceFamily.NAME]
+                name = family[speechserver.VoiceFamily.NAME] \
+                    + " (%s)" % family[speechserver.VoiceFamily.LOCALE]
                 self.acss = acss.ACSS({acss.ACSS.FAMILY : family})
                 self.voiceChoices[i] = self.acss
                 self.voicesModel.append((i, name))
                 i += 1
             self.defaultACSS = self.voiceChoices[1]
         else:
-            name = self.families[0][speechserver.VoiceFamily.NAME]
+            name = self.families[0][speechserver.VoiceFamily.NAME] \
+                + " (%s)" % self.families[0][speechserver.VoiceFamily.LOCALE]
             self.voicesModel.append((1, name))
             self.defaultACSS = \
                 acss.ACSS({acss.ACSS.FAMILY : self.families[0]})
@@ -743,15 +745,15 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
         """
 
         familyName = None
-        family = self._getKeyForVoiceType(voiceType, "family")
+        family = self._getKeyForVoiceType(voiceType, acss.ACSS.FAMILY)
 
         if family:
-            if family.has_key("name"):
-                familyName = family["name"]
+            if family.has_key(speechserver.VoiceFamily.NAME):
+                familyName = family[speechserver.VoiceFamily.NAME]
 
         return familyName
 
-    def _setFamilyNameForVoiceType(self, voiceType, value):
+    def _setFamilyNameForVoiceType(self, voiceType, name, language):
         """Set the name of the voice family for the given voice type.
 
         Arguments:
@@ -759,13 +761,15 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
         - value: the name of the voice family to set.
         """
 
-        family = self._getKeyForVoiceType(voiceType, "family", False)
+        family = self._getKeyForVoiceType(voiceType, acss.ACSS.FAMILY, False)
         if family:
-            family["name"] = value
+            family[speechserver.VoiceFamily.NAME] = name
+            family[speechserver.VoiceFamily.LOCALE] = language
         else:
             voice = self._getVoiceForVoiceType(voiceType)
-            voice["family"] = {}
-            voice["family"]["name"] = value
+            voice[acss.ACSS.FAMILY] = {}
+            voice[acss.ACSS.FAMILY][speechserver.VoiceFamily.NAME] = name
+            voice[acss.ACSS.FAMILY][speechserver.VoiceFamily.LOCALE] = language
 
     def _getRateForVoiceType(self, voiceType):
         """Get the rate value for the given voice type.
@@ -1207,13 +1211,17 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
         Arguments:
         - widget: the component that generated the signal.
         """
-
-        iter = widget.get_active_iter()
-        model = widget.get_model()
-
-        name = model.get_value(iter, 1)
         voiceType = self.voiceType.get_active_text()
-        self._setFamilyNameForVoiceType(voiceType, name)
+        selectedIndex = widget.get_active()
+        try:
+            acss = self.voiceChoices[selectedIndex + 1]
+            family = acss[acss.FAMILY]
+            name = family[speechserver.VoiceFamily.NAME]
+            language = family[speechserver.VoiceFamily.LOCALE]
+            self._setFamilyNameForVoiceType(voiceType, name, language)
+        except:
+            debug.printException(debug.LEVEL_WARNING)
+            pass
 
     def speechIndentationChecked(self, widget):
         """Signal handler for the "toggled" signal for the
