@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2006 Sun Microsystems Inc.
+# Copyright 2005-2007 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 1847 2006-12-19 21:23:59Z wwalker $"
-__version__   = "$Revision: 1847 $"
-__date__      = "$Date: 2006-12-19 16:23:59 -0500 (Tue, 19 Dec 2006) $"
+__id__        = "$Id: speechgenerator.py 1933 2007-01-17 19:32:35Z wwalker $"
+__version__   = "$Revision: 1933 $"
+__date__      = "$Date: 2007-01-17 14:32:35 -0500 (Wed, 17 Jan 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1259,7 +1259,8 @@ class SpeechGenerator:
         utterances = []
 
         if (not already_focused):
-            if settings.readTableCellRow and obj.parent.table:
+            if settings.readTableCellRow and obj.parent.table \
+                and (not self._script.isLayoutOnly(obj.parent)):
                 parent = obj.parent
                 row = parent.table.getRowAtIndex(obj.index)
                 column = parent.table.getColumnAtIndex(obj.index)
@@ -1293,6 +1294,11 @@ class SpeechGenerator:
         else:
             utterances = self._getSpeechForTableCell(obj, already_focused)
 
+        self._debugGenerator("_getSpeechForTableCellRow",
+                             obj,
+                             already_focused,
+                             utterances)
+
         return utterances
 
     def _getSpeechForTableColumnHeader(self, obj, already_focused):
@@ -1561,20 +1567,17 @@ class SpeechGenerator:
                 and (parent.role != rolenames.ROLE_LAYERED_PANE) \
                 and (parent.role != rolenames.ROLE_SPLIT_PANE) \
                 and (parent.role != rolenames.ROLE_SCROLL_PANE) \
-                and (parent.role != rolenames.ROLE_UNKNOWN):
+                and (parent.role != rolenames.ROLE_UNKNOWN) \
+                and (not self._script.isLayoutOnly(parent)):
 
                 # Announce the label and text of the object in the hierarchy.
                 #
                 text = util.getDisplayedText(parent)
                 label = util.getDisplayedLabel(parent)
 
-                # Don't announce unlabelled panels.
+                # Push announcement of cell after text and label.
                 #
-                if parent.role == rolenames.ROLE_PANEL \
-                   and (((not label) or (len(label) == 0) \
-                         or (not text) or (len(text) == 0))):
-                    pass
-                elif parent.role != rolenames.ROLE_TABLE_CELL:
+                if parent.role != rolenames.ROLE_TABLE_CELL:
                     utterances.append(rolenames.getSpeechForRoleName(parent))
 
                 if text and len(text):
