@@ -19,9 +19,9 @@
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py,v 1.7 2006/12/13 18:45:24 lmonsanto Exp $"
-__version__   = "$Revision: 1.7 $"
-__date__      = "$Date: 2006/12/13 18:45:24 $"
+__id__        = "$Id: where_am_I.py 1952 2007-01-21 22:42:11Z wwalker $"
+__version__   = "$Revision: 1952 $"
+__date__      = "$Date: 2007-01-21 17:42:11 -0500 (Sun, 21 Jan 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -634,7 +634,8 @@ def _speakCalc(obj, doubleClick):
     
     # No way to get cell coordinates?
     
-    [textContents, startOffset, endOffset, selected] = _getTextContents(obj)
+    [textContents, startOffset, endOffset, selected] = \
+        _getTextContents(obj, doubleClick)
     text = _("%s") % textContents
     utterances.append(text)
     if selected:
@@ -1260,15 +1261,20 @@ def _speakStatusBar():
         return
 
     utterances = []
-    for i in range(0, _statusBar.childCount):
-        child = _statusBar.child(i)
-        text = _("%s") % _getObjName(child)
+
+    if _statusBar.childCount == 0:
+        text = _("%s") % _getObjName(_statusBar)
         utterances.append(text)
+    else:
+        for i in range(0, _statusBar.childCount):
+            child = _statusBar.child(i)
+            text = _("%s") % _getObjName(child)
+            utterances.append(text)
 
     debug.println(_debugLevel, "statusbar utterances=%s" % \
                   utterances)
     speech.speakUtterances(utterances)
-
+    
     
 def _speakCalcStatusBar():
     """
